@@ -1,12 +1,16 @@
 import os
+import typing as ty
 
 from flask import request
-from typing import Callable
+from typing import Callable, Type
 
 from nsj_rest_lib.controller.controller_util import DEFAULT_RESP_HEADERS
 from nsj_rest_lib.controller.route_base import RouteBase
+from nsj_rest_lib.dao.dao_base import DAOBase
 from nsj_rest_lib.dto.dto_base import DTOBase
+from nsj_rest_lib.dto.queued_data_dto import QueuedDataDTO
 from nsj_rest_lib.entity.entity_base import EntityBase
+from nsj_rest_lib.entity.function_type_base import InsertFunctionTypeBase
 from nsj_rest_lib.exception import (
     MissingParameterException,
     ConflictException,
@@ -31,6 +35,9 @@ class PostRoute(RouteBase):
         handle_exception: Callable = None,
         custom_before_insert: Callable = None,
         custom_after_insert: Callable = None,
+        retrieve_after_insert: bool = False,
+        insert_function_type_class: Type[InsertFunctionTypeBase] | None = None,
+        insert_function_name: str | None = None,
     ):
         super().__init__(
             url=url,
@@ -44,6 +51,37 @@ class PostRoute(RouteBase):
         )
         self.custom_before_insert = custom_before_insert
         self.custom_after_insert = custom_after_insert
+        self.retrieve_after_insert = retrieve_after_insert
+        self._insert_function_type_class = insert_function_type_class
+        self._insert_function_name = insert_function_name
+
+        if self._insert_function_type_class is not None and not issubclass(
+            self._insert_function_type_class, InsertFunctionTypeBase
+        ):
+            raise ValueError(
+                "A classe informada em insert_function_type_class deve herdar de InsertFunctionTypeBase."
+            )
+
+    def _get_service(self, factory: NsjInjectorFactoryBase):
+        """
+        Sobrescreve o _get_service padrão para permitir configurar
+        o InsertFunctionType e o nome da função diretamente no Service.
+        """
+
+        if self._service_name is not None:
+            return factory.get_service_by_name(self._service_name)
+
+        from nsj_rest_lib.service.service_base import ServiceBase
+
+        return ServiceBase(
+            factory,
+            DAOBase(factory.db_adapter(), self._entity_class),
+            self._dto_class,
+            self._entity_class,
+            self._dto_response_class,
+            insert_function_type_class=self._insert_function_type_class,
+            insert_function_name=self._insert_function_name,
+        )
 
     def _partition_filters(self, data):
         # Montando os filtros de particao de dados
@@ -62,6 +100,7 @@ class PostRoute(RouteBase):
         self,
         query_args: dict[str, any] = None,
         body: dict[str, any] = None,
+        **kwargs: ty.Any,
     ):
         """
         Tratando requisições HTTP Post para inserir uma instância de uma entidade.
@@ -78,11 +117,12 @@ class PostRoute(RouteBase):
                 if not isinstance(request_data, list):
                     request_data = [request_data]
 
-
                 data_pack = []
                 lst_data = []
                 partition_filters = None
                 for item in request_data:
+                    if len(kwargs) >= 0:
+                        item.update(kwargs)
 
                     # Convertendo os dados para o DTO
                     data = self._dto_class(validate_read_only=True, **item)
@@ -93,20 +133,36 @@ class PostRoute(RouteBase):
 
                     data_pack.append(data)
 
+                if partition_filters is None:
+                    if len(kwargs) > 0:
+                        partition_filters = kwargs.copy()
+                else:
+                    partition_filters.update(kwargs)
 
                 # Construindo os objetos
                 service = self._get_service(factory)
 
-                if len(data_pack)==1:
+                if len(data_pack) == 1:
                     # Chamando o service (método insert)
                     data = service.insert(
                         dto=data,
                         aditional_filters=partition_filters,
                         custom_before_insert=self.custom_before_insert,
                         custom_after_insert=self.custom_after_insert,
+                        retrieve_after_insert=self.retrieve_after_insert,
+                        function_name=self._insert_function_name,
                     )
 
                     if data is not None:
+                        # Verificando se houve um enfileiramento (pelo custom_after_insert)
+                        if isinstance(data, QueuedDataDTO):
+                            queued_data: QueuedDataDTO = data
+                            resp_headers = {
+                                **DEFAULT_RESP_HEADERS,
+                                "Location": queued_data.status_url,
+                            }
+                            return ("", 202, resp_headers)
+
                         # Convertendo para o formato de dicionário (permitindo omitir campos do DTO)
                         lst_data.append(data.convert_to_dict())
                 else:
@@ -115,17 +171,19 @@ class PostRoute(RouteBase):
                         aditional_filters=partition_filters,
                         custom_before_insert=self.custom_before_insert,
                         custom_after_insert=self.custom_after_insert,
+                        retrieve_after_insert=self.retrieve_after_insert,
+                        function_name=self._insert_function_name,
                     )
 
-                    if data is not None or not len(data)>0:
+                    if data is not None or not len(data) > 0:
                         # Convertendo para o formato de dicionário (permitindo omitir campos do DTO)
                         lst_data = [item.convert_to_dict() for item in data]
 
-                if len(lst_data)==1:
+                if len(lst_data) == 1:
                     # Retornando a resposta da requisição
                     return (json_dumps(lst_data[0]), 200, {**DEFAULT_RESP_HEADERS})
 
-                if len(lst_data)>1:
+                if len(lst_data) > 1:
                     # Retornando a resposta da requisição
                     return (json_dumps(lst_data), 200, {**DEFAULT_RESP_HEADERS})
 
