@@ -11,3 +11,7 @@ class FilterOperator(enum.Enum):
     LIKE = "like"
     ILIKE = "ilike"
     NOT_NULL = "not_null"
+    LENGTH_GREATER_OR_EQUAL_THAN = "length_greater_or_equal_than"
+    LENGTH_LESS_OR_EQUAL_THAN = "length_less_or_equal_than"
+    IN = "in"
+    NULL = "null"
