@@ -8,6 +8,8 @@ from nsj_rest_lib.healthcheck_config import HealthCheckConfig
 HealthCheckConfig(flask_application=application).config(True)
 
 import tests.cliente_controller
+import tests.classificacao_financeira_controller
+import tests.cliente_byfunction_controller
 
 if __name__ == "__main__":
-    application.run(port=5000)
\ No newline at end of file
+    application.run(port=5000)
